import os, sys, unittest
try:
    from zope.testing import doctest
except ImportError:
    import doctest
    
optionflags = doctest.REPORT_ONLY_FIRST_FAILURE | doctest.ELLIPSIS

def test_suite():
    return unittest.TestSuite((
        doctest.DocFileSuite('README.txt', package="memojito",
                             optionflags=optionflags),
        ))

if __name__=="__main__":
    import unittest
    unittest.TextTestRunner().run(test_suite())



"""
see README.txt
"""
_marker = object()
class Memojito(object):
    propname = '_memojito_'
    def clear(self, inst):
        if hasattr(inst, self.propname):
            delattr(inst, self.propname)
        
    def clearbefore(self, func):
        def clear(*args, **kwargs):
            inst=args[0]
            self.clear(inst)
            return func(*args, **kwargs)
        return clear

    def clearafter(self, func):
        def clear(*args, **kwargs):
            inst=args[0]
            val = func(*args, **kwargs)
            self.clear(inst)
            return val 
        return clear

    def memoizedproperty(self, func):
        return property(self.memoize(func))

    def memoize(self, func):
        def memogetter(*args, **kwargs):
            inst = args[0]
            cache = getattr(inst, self.propname, dict())

            # XXX this could be improved to unfold unhashables
            # and optimized with pyrex
            
            key = (func.__name__, args, frozenset(kwargs.items()))
            key=hash(key)
            val = cache.get(key, _marker)
            if val is _marker:
                val=func(*args, **kwargs)
                cache[key]=val
                setattr(inst, self.propname, cache)
            return val
        return memogetter

_m = Memojito()
memoize = _m.memoize
memoizedproperty = _m.memoizedproperty
clearbefore = _m.clearbefore
clearafter = _m.clearafter



import os, sys
from ez_setup import use_setuptools
use_setuptools()

from setuptools import setup, find_packages

setup(
    name = 'memojito',
    version = '0.6',
    url='http://svn.plone.org/svn/collective/plonents/components/memojito/trunk#egg=memojito',
    license='MIT',
    description='simple library of memoizing decorators',
    author='Whit Morriss',
    author_email='whit@openplans.org',
    packages = find_packages('src'),
    package_dir = {'': 'src'},
    include_package_data = True,
    zip_safe = True)
